/*

shieldequalizercapacitors.js

Script for shield equalizer and shield capacitor equipment for your ship.


Oolite
Copyright  2003-2010 Giles C Williams and contributors

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
MA 02110-1301, USA.


shieldequalizercapacitors.oxp
Copyright  2011 "CommonSenseOTB"

This work is licensed under the Creative Commons
Attribution-Noncommercial-Share Alike 3.0 Unported License.

To view a copy of this license, visit
http://creativecommons.org/licenses/by-nc-sa/3.0/ or send a letter
to Creative Commons, 171 Second Street, Suite 300, San Francisco,
California, 94105, USA.

*/

this.name        = "shieldequalizercapacitors";
this.author      = "CommonSenseOTB";
this.copyright   = "Copyright 2011 by CommonSenseOTB CC-NC-by-SA 3.0";
this.description = "Provides shield equalizer and shield capacitor equipment for your ship.";
this.version     = "1.7";

this.ship;
this.$debug = false;
this.$logging = true;
this.$maxCapacitorCharge = 64;
this.$shieldEqualizerCounter = 0;
this.$NPC_enabled;
this.$spawnedShipsTimers = []; // will hold timer references for the spawned ships
this.$npcShieldEnabingOXPs = [  // one worldScript name of each OXP that provides shield functionality for NPC
                                // the OXP MUST do that by creating and using aftShield and forwardShield properties 
                                // in the NPC ship object
                                'NShields',
                             ];  
this.$libConfig = {
    Name:   "shieldequalizercapacitors",
    Display:"Config",
    Alive:  "$libConfig",
    Bool:   {
                B0: {
                    Name:   "$NPC_enabled",
                    Desc:   "enabled for NPCs",
                    Def:    false,
                    Notify: "$updateMissionVars",
                },
                Info: "enable/disable Shield Equalizer and Capacitors for NPCs: false=disabled, true=enabled"
    }
}


//
// Event Handlers
//

//----------------------------------------------------------------------------------------------//
this.startUp = function _startUp() {
    // You may toggle the sound and messages here
    // Developers may have their hud script toggle these remotely
    //
    // YOU MAY TOUCH THESE
    //
    this.togglesoundfx1 = "ON";// default "ON", change to "OFF" to turn off soundfx1(energy transfer sounds)(1,2)
    this.togglesoundfx2 = "ON";// default "ON", change to "OFF" to turn off soundfx2(countdown timer and alarm)(3,4,5)
    this.togglemessages = "ON";// default "ON", change to "OFF" to turn off messages(all)
    //
    // DO NOT TOUCH THESE PLEASE
    //
    var mv = missionVariables;
    if (!mv.shieldequalizercapacitors_NPC_enabled) mv.shieldequalizercapacitors_NPC_enabled = 0;
    delete mv.secforwardshieldcapacitor;
    delete mv.secaftshieldcapacitor;
    this.$NPC_enabled = (mv.shieldequalizercapacitors_NPC_enabled == 1);
    this.secforwardshieldcapacitor = 0.0;
    this.secaftshieldcapacitor = 0.0;
    this.fscpause = 1.0;
    this.ascpause = 1.0;
    this.mySound1 = new SoundSource;
    this.mySound1.sound = "ShieldCapacitorDischarge.ogg";
    this.mySound1.loop = false;
    this.mySound2 = new SoundSource;
    this.mySound2.sound = "ShieldEqualizerEqualizing.ogg";
    this.mySound2.loop = false;
    this.mySound3 = new SoundSource;
    this.mySound3.sound = "ShieldCapacitorCharging.ogg";
    this.mySound3.loop = false;
    this.mySound3.repeatCount = 32;// bugfix v1.1
    this.mySound4 = new SoundSource;
    this.mySound4.sound = "ShieldCapacitorCharging.ogg";
    this.mySound4.loop = false;
    this.mySound4.repeatCount = 32;// bugfix v1.1
    this.mySound5 = new SoundSource;
    this.mySound5.sound = "ShieldCapacitorCharged.ogg";
    this.mySound5.loop = false;
    this.ship = player.ship;
}

//----------------------------------------------------------------------------------------------//
this.startUpComplete = function _startUpComplete() {
    var npcEnablingOXPs = this.$npcShieldEnabingOXPs;
    var hasNPCshieldEnablingOXP = false;
    var i = npcEnablingOXPs.length;
    while (i--) 
        if (worldScripts[npcEnablingOXPs[i]]) {
            hasNPCshieldEnablingOXP = true;
            break;
        }
    if (!hasNPCshieldEnablingOXP) {
        this.$NPC_enabled = false;
        log(this.name, "No OXP that enables shields for NPC was found, disabling NPC functionality")
    }  else if (worldScripts.Lib_Config) {
        var ret = worldScripts.Lib_Config._registerSet(this.$libConfig);
        if (this.$debug) log(this.name, "Lib Config registerSet returned "+ret);
    }  
}

//----------------------------------------------------------------------------------------------//
this.playerBoughtEquipment = function _playerBoughtEquipment(equipmentKey)   {
    var eqInfo;
    var ship = player.ship;

    if (equipmentKey === "EQ_FORWARD_SHIELD_CAPACITOR_REMOVAL") {
        ship.removeEquipment("EQ_FORWARD_SHIELD_CAPACITOR");
        ship.removeEquipment("EQ_FORWARD_SHIELD_CAPACITOR_REMOVAL");
        player.credits += 0.9 * EquipmentInfo.infoForKey(equipmentKey).price;
    } else if (equipmentKey === "EQ_AFT_SHIELD_CAPACITOR_REMOVAL") {
        ship.removeEquipment("EQ_AFT_SHIELD_CAPACITOR");
        ship.removeEquipment("EQ_AFT_SHIELD_CAPACITOR_REMOVAL");
        player.credits += 0.9 * EquipmentInfo.infoForKey(equipmentKey).price;
    } else if (equipmentKey === "EQ_SHIELD_EQUALIZER_REMOVAL") {
        ship.removeEquipment("EQ_SHIELD_EQUALIZER");
        ship.removeEquipment("EQ_SHIELD_EQUALIZER_REMOVAL");
        player.credits += 0.9 * EquipmentInfo.infoForKey(equipmentKey).price;
    }
}

//----------------------------------------------------------------------------------------------//
this.equipmentDamaged = this.equipmentDestroyed = function _equipmentDamaged(equipment) {
    var ship = this.ship;
    // due to the shield equalizer's high quality cross-connected energy transfer power couplings
    // and robust construction there is more resistance to damage for the SHIELD system in general
    // ALTER DAMAGE CHANCE FOR THE FOLLOWING ITEMS
    if (equipment === "EQ_SHIELD_EQUALIZER") {
        if(Math.random() > 0.5)
            ship.setEquipmentStatus("EQ_SHIELD_EQUALIZER","EQUIPMENT_OK");
    } else if (equipment === "EQ_FORWARD_SHIELD_CAPACITOR") {
        if (ship.equipmentStatus("EQ_SHIELD_EQUALIZER") === "EQUIPMENT_OK") {
            if(Math.random() > 0.5)
                ship.setEquipmentStatus("EQ_FORWARD_SHIELD_CAPACITOR","EQUIPMENT_OK");
        } else if (Math.random() > 0.75) {
            if(this.togglesoundfx1 == "ON")
                this.mySound1.play();
            ship.forwardShield -= 0.5 * this.$maxCapacitorCharge;
            log(this.name, ship.displayName+": Forward Shield Capacitor Feedback Damage!!!", 6);
            player.consoleMessage("Forward Shield Capacitor Feedback Damage!!!", 6);
        }
    }else if (equipment === "EQ_AFT_SHIELD_CAPACITOR") {
        if (ship.equipmentStatus("EQ_SHIELD_EQUALIZER") === "EQUIPMENT_OK") {
            if (Math.random() > 0.5)
                ship.setEquipmentStatus("EQ_AFT_SHIELD_CAPACITOR","EQUIPMENT_OK");
        } else if (Math.random() > 0.75) {
            if(this.togglesoundfx1 == "ON")
               this.mySound1.play();
            ship.aftShield -= 0.5 * this.$maxCapacitorCharge;
            log(this.name, ship.displayName+": Aft Shield Capacitor Feedback Damage!!!", 6);
            player.consoleMessage("Aft Shield Capacitor Feedback Damage!!!", 6);
        }
    } else if (equipment === "EQ_SHIELD_BOOSTER") {
        if (ship.equipmentStatus("EQ_SHIELD_EQUALIZER") === "EQUIPMENT_OK") {
            if (Math.random() > 0.65)
                ship.setEquipmentStatus("EQ_SHIELD_BOOSTER","EQUIPMENT_OK");
        }
    } else if (equipment === "EQ_NAVAL_SHIELD_BOOSTER") {
        if (ship.equipmentStatus("EQ_SHIELD_EQUALIZER") === "EQUIPMENT_OK") {
            if (Math.random() > 0.65)
                ship.setEquipmentStatus("EQ_NAVAL_SHIELD_BOOSTER","EQUIPMENT_OK");
        }
    } else if (equipment === "EQ_NAVAL_GRID") {
        if (ship.equipmentStatus("EQ_SHIELD_EQUALIZER") === "EQUIPMENT_OK") {
            if (Math.random() > 0.35)
                ship.setEquipmentStatus("EQ_NAVAL_GRID","EQUIPMENT_OK");
        }
    } else if (equipment === "EQ_ENERGY_GRID") {
        if (ship.equipmentStatus("EQ_SHIELD_EQUALIZER") === "EQUIPMENT_OK") {
            if (Math.random() > 0.25)
                ship.setEquipmentStatus("EQ_ENERGY_GRID","EQUIPMENT_OK");
        }
    }
}

//----------------------------------------------------------------------------------------------//
this.shipSpawned = function _shipSpawned(ship) {
    if (!this.$NPC_enabled || ship.isBoulder || ship.isCargo || ship.isRock) return;
    var _spawnedShipsTimers = this.$spawnedShipsTimers;
    var id = _spawnedShipsTimers.length; // array indices start at 0
    // give time for other OXPs (EscortDeck!) to change the ship's script before hooking things into it
    // give time for other OXPs to arward the Shield Equalizer and/of Shield Capacitors
    this.$spawnedShipsTimers[id] = new Timer(this, this.$setSpawnedShip.bind(this, id, ship), 2+2*Math.random(), 0);
}

//----------------------------------------------------------------------------------------------//
this.shipLaunchedFromStation = function _shipLaunchedFromStation() {
    this.ship = player.ship; // just in case player changed ships while docked
    if (this.$shieldCheckTimer)
        this.$shieldCheckTimer.start();
    else
        this.$shieldCheckTimer = new Timer(this, this.$shieldCheck, 0, 0.5);// minimum 0.5 seconds for equalizer to react
}

//----------------------------------------------------------------------------------------------//
this.shipWillEnterWitchspace = function _shipWillEnterWitchspace() {
    var _spawnedShipsTimers = this.$spawnedShipsTimers;
    var i = _spawnedShipsTimers.length;
    while (i--) {
        if (_spawnedShipsTimers[i] && _spawnedShipsTimers[i].isRunning)
            _spawnedShipsTimers[i].stop();
        delete _spawnedShipsTimers[i];
    }
    _spawnedShipsTimers.length = 0;
}

//----------------------------------------------------------------------------------------------//
this.shipDied = function _shipDied() {
    var ship = this.ship;
    var npcShip = (ship != player.ship);

    if (this.$shieldCheckTimer) this.$shieldCheckTimer.stop()
    if (npcShip) return;
    if (this.mySound3.isPlaying) this.mySound3.stop();
    if (this.mySound4.isPlaying) this.mySound4.stop();
}    


//
// Internal Functions
//

//----------------------------------------------------------------------------------------------//
this.$updateMissionVars = function _updateMissionVars() { // called by Lib_Config when user changes things
    missionVariables.shieldequalizercapacitors_NPC_enabled = (this.$NPC_enabled === true ? 1 : 0);
    if (this.$debug) log(this.name, "missionVariable shieldequalizercapacitors_NPC_enabled updated to "+missionVariables.shieldequalizercapacitors_NPC_enabled);
}

//----------------------------------------------------------------------------------------------//
this.$setSpawnedShip = function _setSpawnedShip(id, ship) { // hooks equipment porperties and functions into ship's script
    var wc = worldScripts.shieldequalizercapacitors;
    var _spawnedShipsTimers = this.$spawnedShipsTimers;
    if (ship.aftShield != null && // has functional shields (provided by some OXP...)
        (ship.equipmentStatus("EQ_SHIELD_EQUALIZER") === "EQUIPMENT_OK" || 
         ship.equipmentStatus("EQ_FORWARD_SHIELD_CAPACITOR") === "EQUIPMENT_OK" ||
         ship.equipmentStatus("EQ_AFT_SHIELD_CAPACITOR") === "EQUIPMENT_OK") ) {
        // functions
        ship.script.$shieldCheck = wc.$shieldCheck;
        ship.script.$shieldequalizer = wc.$shieldequalizer;
        ship.script.$capacitorRecharge = wc.$capacitorRecharge;
        ship.script.$dischargeCapacitor = wc.$dischargeCapacitor;
        ship.script.$forwardshieldcapacitor = wc.$forwardshieldcapacitor;
        ship.script.$aftshieldcapacitor = wc.$aftshieldcapacitor;
        ship.script.shipDied = wc.shipDied;
        // properties
        ship.script.secforwardshieldcapacitor = 0.0;
        ship.script.secaftshieldcapacitor = 0.0;
        ship.script.$maxCapacitorCharge = this.$maxCapacitorCharge;
        if (!ship.script.$debug) ship.script.$debug = false;
        if (!ship.script.$logging) ship.script.$logging = false;
        ship.script.$shieldEqualizerCounter = 0;
        if (ship.script.$EscortDeckUsable === 1) ship.script.$logging = true; // enable logging fot player's escorts
        // main timer
        ship.script.$shieldCheckTimer = new Timer(ship.script, ship.script.$shieldCheck, 0, 0.5);// minimum 0.5 seconds for equalizer to react
        if (this.$logging) log (this.name, ship.displayName+": Enabling Shield Equalizer and Capacitors");
    }
    delete _spawnedShipsTimers[id];
    _spawnedShipsTimers[id] = null;

    var i = _spawnedShipsTimers.length;
    var count = 0;
    while (i--) if (_spawnedShipsTimers[i]) count++;
    if (count === 0) {
        // no active timers, clean the array
        if (this.$debug) log(this.name, "No more NPC setup timers running, cleaning the timers array");
        _spawnedShipsTimers.length = 0;
    } else if (this.$debug) log(this.name, count+" NPC setup timers still running");
}

//----------------------------------------------------------------------------------------------//
this.$shieldCheck = function _shieldCheck() { // NPC-enabled: works both for player ship and NPCs (if they have it in their ship.script)
    var ship = this.ship;
    var npcShip = (ship != player.ship);

    if (ship.alertCondition === 0) { // Docked
        this.secforwardshieldcapacitor = 0.0;// discharge capacitors for safety when docked
        this.secaftshieldcapacitor = 0.0;
        this.$shieldCheckTimer.stop();
        if (!npcShip) {
            if (this.mySound3.isPlaying) this.mySound3.stop();
            if (this.mySound4.isPlaying) this.mySound4.stop();
        }
    } else { // main script functions
        this.$shieldEqualizerCounter += 1;
        this.$capacitorRecharge();
        this.$forwardshieldcapacitor();
        this.$aftshieldcapacitor();
        this.$shieldequalizer();
        if (this.$debug && this.$shieldEqualizerCounter % 120 === 0) {
            log("shieldequalizercapacitors", ship.displayName+": shields: "+ship.forwardShield.toFixed(1)+"/"+ship.aftShield.toFixed(1)+", Capacitors: "+this.secforwardshieldcapacitor+"/"+this.secaftshieldcapacitor+", Energy: "+ship.energy.toFixed(0));
            this.$shieldEqualizerCounter = 0;
        }
    }
}

//----------------------------------------------------------------------------------------------//
this.$shieldequalizer = function _shieldequalizer() { // NPC-enabled: works both for player ship and NPCs (if they have it in their ship.script)
    var ship = this.ship;
    var npcShip = (ship != player.ship);
    var logging = this.$logging;

    function equalizeShields(from, to) {
        if (!npcShip && this.togglesoundfx1 === "ON") 
            this.mySound2.play();
        if (ship[from] - ship[to] > 50.0) {// make transfer smooth
                if (logging) log("shieldequalizercapacitors", ship.displayName+": shields: "+ship.forwardShield.toFixed(1)+"/"+ship.aftShield.toFixed(1)+", transferring 25 units");
                ship[from] -= 25.0;
                ship[to] += 25.0;
            } else if (ship[from] - ship[to] > 10.0) {
                if (logging) log("shieldequalizercapacitors", ship.displayName+": shields: "+ship.forwardShield.toFixed(1)+"/"+ship.forwardShield.toFixed(1)+", transferring 5 units");
                ship[from] -= 5.0;
                ship[to] += 5.0;
            } else {
                if (logging) log("shieldequalizercapacitors", ship.displayName+": shields: "+ship.forwardShield.toFixed(1)+"/"+ship.aftShield.toFixed(1)+", transferring 1 unit");
                ship[from] -= 1.0;
                ship[to] += 1.0;
                if (ship[from] - ship[to] <= 2.0) {
                    if (logging) log("shieldequalizercapacitors", ship.displayName+": shields: "+ship.forwardShield.toFixed(1)+"/"+ship.aftShield.toFixed(1)+", equalizing");
                    ship[to] = (ship[from] + ship[to]) / 2;// averaging the shields prevents problems
                    ship[from] = ship[to];
                }
            }
    }

    if (ship.equipmentStatus("EQ_SHIELD_EQUALIZER") === "EQUIPMENT_OK" && (npcShip || !ship.torusEngaged)) {
        // Shield Equalizer functional and (NPC or (player ship not in Torus))
        if (Math.abs(ship.forwardShield - ship.aftShield) > 0.1) {
            if (ship.forwardShield - ship.aftShield > 0)
                // transfer from forward shield to aft shield
                equalizeShields("forwardShield", "aftShield");
            else
                //transfer from aft shield to forward shield
                equalizeShields("aftShield", "forwardShield");
        }
    }
}

//----------------------------------------------------------------------------------------------//
this.$capacitorRecharge = function _capacitorRecharge() {
    var that = _capacitorRecharge;
    var ship = this.ship;
    var pship = player.ship;
    var npcShip = (ship != pship);
    var logging = (this.$logging || ship == pship.target);
    var capacitor_data  = (that.capacitor_data = that.capacitor_data || {
                                                "aft":  {
                                                            capacitor: "secaftshieldcapacitor",
                                                            shield: "aftShield",
                                                            maxShield: "maxAftShield",
                                                            rechargeRate: "aftShieldRechargeRate"
                                                        },
                                                "fwd":  {
                                                            capacitor: "secforwardshieldcapacitor",
                                                            shield: "forwardShield",
                                                            maxShield: "maxForwardShield",
                                                            rechargeRate: "forwardShieldRechargeRate"
                                                        }
                                                                        });
    var maxCharge = this.$maxCapacitorCharge;
    
    function chargeCapacitor(capacitor, context) {
        var data = capacitor_data[capacitor];
        if (context[data.capacitor] < maxCharge &&    // capacitor not full
            (ship.torusEngaged || ship[data.maxShield] - ship[data.shield] < 0.1) // shield full or ship in Torus
           ) {
            // capacitor not full, charge it
            let old_energy = ship.energy;
            let energy_transfer = ship[data.rechargeRate];
            if (isNaN(energy_transfer))
                log("shieldequalizercapacitors", ship.displayName+": "+capacitor+" capacitor recharge failed, invalid recharge value: "+energy_transfer);
            else {
                ship.energy -= energy_transfer;
                if (logging && context.$shieldEqualizerCounter % 10 === 0) log("shieldequalizercapacitors", ship.displayName+": "+capacitor+" capacitor: "+context[data.capacitor].toFixed(1)+", charging: "+energy_transfer.toFixed(1)+", shipEnergy: "+old_energy.toFixed(1)+"->"+ship.energy.toFixed(1));
                let capacitor_charge = context[data.capacitor] + energy_transfer;
                if (!npcShip) {
                    if (capacitor_charge < maxCharge) {
                        if (context.togglemessages === "ON") {
                            context.fscpause += 1.0;
                            if (context.fscpause > 3.0) 
                                context.fscpause = 1.0;
                            if (context.fscpause == 2.0) 
                                log("shieldequalizercapacitors", ship.displayName+": "+capacitor+" capacitor ===> CHARGING...");
                        }            
                        if (context.togglesoundfx2 === "ON") {
                           if (!context.mySound3.isPlaying) 
                                context.mySound3.play();
                        }
                    } else if (capacitor_charge >= maxCharge) {
                        if (context.togglemessages === "ON") 
                            log("shieldequalizercapacitors", ship.displayName+": "+capacitor+" capacitor ===> FULLY CHARGED");
                        if (context.mySound3.isPlaying) 
                            context.mySound3.stop();
                        if (context.togglesoundfx2 === "ON") 
                            context.mySound5.play();
                    }
                }            
                if (capacitor_charge > maxCharge)
                    capacitor_charge = maxCharge;
                context[data.capacitor] = capacitor_charge;
            }
        }
    }

    if (ship.equipmentStatus("EQ_FORWARD_SHIELD_CAPACITOR") === "EQUIPMENT_OK")
        chargeCapacitor("fwd", this);
    if (ship.equipmentStatus("EQ_AFT_SHIELD_CAPACITOR") === "EQUIPMENT_OK") 
        chargeCapacitor("aft", this);
}

//----------------------------------------------------------------------------------------------//
this.$forwardshieldcapacitor = function _forwardshieldcapacitor() { // NPC-enabled: works both for player ship and NPCs (if they have it in their ship.script)
    if (this.ship.equipmentStatus("EQ_FORWARD_SHIELD_CAPACITOR") === "EQUIPMENT_OK")
        this.$dischargeCapacitor("fwd");
}

//----------------------------------------------------------------------------------------------//
this.$aftshieldcapacitor = function _aftshieldcapacitor() { // NPC-enabled: works both for player ship and NPCs (if they have it in their ship.script)
    if (this.ship.equipmentStatus("EQ_AFT_SHIELD_CAPACITOR") === "EQUIPMENT_OK")
        this.$dischargeCapacitor("aft");
}

//----------------------------------------------------------------------------------------------//
this.$dischargeCapacitor = function _dischargeCapacitor(capacitor) {
    var that = _dischargeCapacitor;
    var ship = this.ship;
    var npcShip = (ship != player.ship);
    var capacitor_data  = (that.capacitor_data = that.capacitor_data || {
                                                "aft":  {
                                                            capacitor: "secaftshieldcapacitor",
                                                            shield: "aftShield",
                                                            maxShield: "maxAftShield",
                                                            rechargeRate: "aftShieldRechargeRate"
                                                        },
                                                "fwd":  {
                                                            capacitor: "secforwardshieldcapacitor",
                                                            shield: "forwardShield",
                                                            maxShield: "maxForwardShield",
                                                            rechargeRate: "forwardShieldRechargeRate"
                                                        }
                                                                        });
    var data = capacitor_data[capacitor];
    if (!ship.torusEngaged && ship.fuel >= 0.1 && this[data.capacitor] > 0) {
        // capacitor has some charge and ship not in Torus Drive, ship has fuel (i.e., not in HardWay emergency mode): see if shields should be recharged
        let topping_charge = ship[data.maxShield] - (ship[data.shield] + ship[data.rechargeRate]);
        if (topping_charge > 0) { // shield not full, transfer energy from capacitor
            if (topping_charge > this[data.capacitor]) 
                topping_charge = this[data.capacitor];
            if (this.$logging) log("shieldequalizercapacitors", ship.displayName+": "+capacitor+" Shield: "+ship[data.shield].toFixed(1)+", transferring "+topping_charge.toFixed(1)+" from capacitor, remaining charge "+(this[data.capacitor] - topping_charge).toFixed(1));
            ship[data.shield] += topping_charge;
            this[data.capacitor] -= topping_charge;
            if (!npcShip) {
                if (this.mySound3.isPlaying) 
                    this.mySound3.stop();
                if (this[data.capacitor]< 0.1 && this.togglemessages === "ON") 
                    log("shieldequalizercapacitors", ship.displayName+": "+capacitor+" Shield Capacitor DISCHARGED!!!");
                if (this.togglesoundfx1 === "ON") 
                    this.mySound1.play();
            }
        }
    }
}

